#! /bin/sh

# assumes UCSD https distro dir is gold copy

pwd=`pwd`
tail=`basename $pwd`
fullpar=`dirname $pwd`
par=`basename $fullpar`
echo $tail
echo $par
if [ "tail" != "scripts" ] || [ "$par" != fsaverage-ADDITIONS ]; then
  echo \
  "zz-getREADME.sh: ### N.B.: not in scripts dir in subject fsaverage-ADDITIONS"
fi

set -x

labeldir=https://pages.ucsd.edu/~msereno/csurf/fsaverage-labels

curl -O $labeldir/README.txt
mv README.txt README1-general.txt

curl -O $labeldir/CsurfMaps1-atlas-movies/README.txt
mv README.txt README2-movies.txt

curl -O $labeldir/CsurfMaps1-illustrations/README.txt
mv README.txt README3-illus.txt

curl -O $labeldir/CsurfMaps1-parcellation/README.txt
mv README.txt README4-parcellation.txt

curl -O $labeldir/CsurfMaps1-parcellation/mapdata/README.txt
mv README.txt README4a-vtxcols.txt

curl -O $labeldir/CsurfMaps1-parcellation/mapdata-labels/README.txt
mv README.txt README4b-mapdata.txt

curl -O $labeldir/CsurfMaps1-parcellation/simple-annot/README.txt
mv README.txt README4c-simpleannot.txt

curl -O $labeldir/bin/README.txt
mv README.txt README5-update.txt

